package com.zskk.dicom.monitor.utils;

import ch.qos.logback.classic.Level;
import ch.qos.logback.classic.Logger;
import ch.qos.logback.classic.LoggerContext;
import ch.qos.logback.classic.encoder.PatternLayoutEncoder;
import ch.qos.logback.classic.filter.LevelFilter;
import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.core.rolling.RollingFileAppender;
import ch.qos.logback.core.rolling.SizeAndTimeBasedRollingPolicy;
import ch.qos.logback.core.spi.FilterReply;
import ch.qos.logback.core.util.FileSize;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.charset.Charset;

/**
 * log4j
 * 
 */
public class LogEnginer {

	/**
	 * 单个文件最大大小(500M)
	 */
	private static FileSize MaxFileSize = FileSize.valueOf("500MB");

	/**
	 * (全部)日志最大大小限制
	 */
	private static FileSize TotalSizeCap = FileSize.valueOf("100GB");

	private static String AppenderFile = "./logs/{logName}.log";

	private static String FileNamePattern = "./logs/%d{yyyy-MM-dd}/{logName}-%d{yyyy-MM-dd}.%i.log";

	/**
	 * 日志输出对象
	 */
	private Logger log;

	/**
	 * 日志Appender配置
	 */
	private RollingFileAppender<ILoggingEvent> appender = new RollingFileAppender<ILoggingEvent>();

	private LoggerContext lc;

	/**
	 * 日志(文件)名
	 */
	String logName;

	@SuppressWarnings({ "rawtypes", "unchecked", "unused" })
	public LogEnginer(String logName) {

		org.slf4j.Logger slog = (org.slf4j.Logger) LoggerFactory.getLogger(logName);

		Object fobj = LoggerFactory.getILoggerFactory();
		if (fobj instanceof org.slf4j.helpers.SubstituteLoggerFactory) {
			org.slf4j.helpers.SubstituteLoggerFactory slf = (org.slf4j.helpers.SubstituteLoggerFactory) fobj;
		} else {
			lc = (LoggerContext) fobj;
		}

		// lc.reset(); // 重置之前的配置（注意：全局）

		log = (Logger) slog;
		log.detachAndStopAllAppenders();

		this.logName = logName;

		appender.setFile(AppenderFile.replaceAll("\\{logName\\}", logName));

		SizeAndTimeBasedRollingPolicy policy = new SizeAndTimeBasedRollingPolicy();
		policy.setContext(lc);
		policy.setFileNamePattern(FileNamePattern.replaceAll("\\{logName\\}", logName));
		policy.setMaxHistory(Integer.MAX_VALUE);
		policy.setTotalSizeCap(TotalSizeCap);
		policy.setMaxFileSize(MaxFileSize);
		policy.setParent(appender);
		policy.start();

		appender.setTriggeringPolicy(policy);
		appender.setRollingPolicy(policy);

		appender.setAppend(true); // 可追加

		PatternLayoutEncoder layoutEncoder = new PatternLayoutEncoder();
		layoutEncoder.setCharset(Charset.forName("utf8"));
		// layoutEncoder.setPattern("%-5p [%d] %C:%L - %m %n");
		layoutEncoder.setPattern("%m%n");
		layoutEncoder.setContext(lc);
		layoutEncoder.start();
		appender.setEncoder(layoutEncoder);

		LevelFilter filter = new LevelFilter();
		filter.setLevel(Level.INFO);
		filter.setOnMatch(FilterReply.ACCEPT);
		filter.setOnMismatch(FilterReply.DENY);

		appender.setName(logName + ".STDOUT");

		// System.out.println("......................");
		// StatusPrinter.printInCaseOfErrorsOrWarnings(lc);
		// System.out.println("......................");

		appender.setContext(lc);
		log.addAppender(appender);
		log.setLevel(Level.ALL);
		log.setAdditive(false);
		appender.start();
	}

	/**
	 * info:一般信息
	 * 
	 * @param logMsg
	 * @throws IOException
	 */
	public synchronized void info(String logMsg) {
		System.out.println(logMsg);
		log.info(logMsg);
	}

	/**
	 * error:出错信息
	 * 
	 * @param logMsg
	 * @throws IOException
	 */
	public synchronized void error(String logMsg) {
		log.error(logMsg);
	}

	/**
	 * debug:调试信息
	 * 
	 * @param logMsg
	 * @throws IOException
	 */
	public synchronized void debug(String logMsg) {
		log.debug(logMsg);
	}

	/**
	 * trace:追踪信息
	 * 
	 * @param logMsg
	 * @throws IOException
	 */
	public synchronized void trace(String logMsg) {
		log.trace(logMsg);
	}

	/**
	 * warn:警告信息
	 * 
	 * @param logMsg
	 * @throws IOException
	 */
	public synchronized void warn(String logMsg) {
		log.warn(logMsg);
	}

}
