package com.zskk.dicom.monitor.utils;

import com.zskk.dicom.monitor.config.Configs;

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class MonitorFileUtils {

	/**
	 * 移除空的目录
	 * 
	 * @param dirFile
	 */
	public static void removeEmptyDir(File dirFile) {
		// 文件的上级目录
		String fs[] = dirFile.list();
		if (fs == null || fs.length < 1) {
			String filePath = dirFile.getAbsolutePath();
			Boolean delRs = dirFile.delete();
			if (delRs) {
				Configs.sysLog.info("Dir is delete suc:" + filePath);
			} else {
				Configs.sysLog.info("Dir is delete fail:" + filePath);
			}
		}
	}
          //移除今天之前的空目录
	public static void removeBeforeTodayEmptyDir(File dirFile) {
		// 文件的上级目录
		String fs[] = dirFile.list();
		if (fs == null || fs.length < 1) {
			DateFormat df = new SimpleDateFormat("yyyyMMdd");
			Calendar ca = Calendar.getInstance();
			ca.setTimeInMillis(dirFile.lastModified());
			String fileDay = df.format(ca.getTime());
			String curDay = df.format(new Date());
			if (!fileDay.equals(curDay)) {
				String filePath = dirFile.getAbsolutePath();
				// 不是同一天
				Boolean delRs = dirFile.delete();
				if (delRs) {
					Configs.sysLog.info("Dir is delete suc:" + filePath);
				} else {
					Configs.sysLog.info("Dir is delete fail:" + filePath);
				}
			}
		}
	}

	/**
	 * 检测备份目录并返回 备份目标 文件(路径)
	 * 
	 * @param dcomFile
	 * @return
	 */
	public static String touchBackDir(File dcomFile) {
		String targetFile = dcomFile.getAbsolutePath();
		targetFile = targetFile.replace(Configs.monitorDir, "");
		targetFile = Configs.monitorBackDir + targetFile;
		String targetFileDirStr = targetFile.substring(0, targetFile.lastIndexOf(File.separator));
		Configs.sysLog.info("64-----64-----错误文件：" + targetFile + "/r/n 目录名称:" + targetFileDirStr);
		File targetFileDir = new File(targetFileDirStr);
		if (!targetFileDir.exists()) {
			Boolean cr = targetFileDir.mkdirs();
			if (cr) {
				Configs.sysLog.info("Create backup dir :" + targetFileDir.getAbsolutePath());
			} else {
				Configs.sysLog.info("Create backup dir failed :" + targetFileDir.getAbsolutePath());
			}
		}
		return targetFile;
	}
	/**
	 * 检测备份目录无创建并返回 重试目标 文件(路径)
	 *
	 * @param dcomFile
	 * @return
	 */
	public static String touchRetryDir(File dcomFile) {
		String targetFile = dcomFile.getAbsolutePath();
		targetFile = targetFile.replace(Configs.monitorDir, "");
		targetFile = Configs.monitorRetryDir + targetFile;
		String targetFileDirStr = targetFile.substring(0, targetFile.lastIndexOf(File.separator));
		Configs.sysLog.info("11-----11-----备份文件：" + targetFile + "/r/n 目录名称:" + targetFileDirStr);
		File targetFileDir = new File(targetFileDirStr);
		if (!targetFileDir.exists()) {
			System.out.println("------------------------------------------------------------------------------------------");
			Boolean cr = targetFileDir.mkdirs();
			if (cr) {
				Configs.sysLog.info("Create backup dir :" + targetFileDir.getAbsolutePath());
			} else {
				Configs.sysLog.info("Create backup dir failed :" + targetFileDir.getAbsolutePath());
			}
		}
		return targetFile;
	}
	/**
	 * 检测备份目录无创建并返回 错误目标 文件(路径)
	 *
	 * @param dcomFile
	 * @return
	 */
	public static String touchEorrDir(File dcomFile) {
		String targetFile = dcomFile.getAbsolutePath();
		targetFile = targetFile.replace(Configs.monitorDir, "");
		targetFile = Configs.eorrRetryDir + targetFile;
		String targetFileDirStr = targetFile.substring(0, targetFile.lastIndexOf(File.separator));
		Configs.sysLog.info("11-----11-----备份文件：" + targetFile + "/r/n 目录名称:" + targetFileDirStr);
		File targetFileDir = new File(targetFileDirStr);
		if (!targetFileDir.exists()) {
			System.out.println("------------------------------------------------------------------------------------------");
			Boolean cr = targetFileDir.mkdirs();
			if (cr) {
				Configs.sysLog.info("Create backup dir :" + targetFileDir.getAbsolutePath());
			} else {
				Configs.sysLog.info("Create backup dir failed :" + targetFileDir.getAbsolutePath());
			}
		}
		return targetFile;
	}

}
